/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "entities.h"
#include "overworld/overworld.h"
#include "assert/assert.h"
#include "util/memory.h"

shaderbuffer_t ENTITIES_BUFFER;
entitiesdata_t ENTITIES_DATA;

void entitiesInit() {
  memset(&ENTITIES_DATA, 0, sizeof(entitiesdata_t));
  shaderBufferInit(&ENTITIES_BUFFER, sizeof(overworld_t));
}

void entitiesUpdate() {
  shaderBufferBind(&ENTITIES_BUFFER);
  shaderBufferSetData(&ENTITIES_BUFFER, &OVERWORLD);

  for(uint8_t i = 0; i < OVERWORLD.entityCount; i++) {
    // Pack the entity data into the buffer
    entity_t *src = &OVERWORLD.entities[i];
    entitiesdataent_t *dst = &ENTITIES_DATA.entities[i];

    // Copy position data.
    memoryCopyRangeSafe(
      &dst->position,
      &src->x,
      &src->subY + sizeof(uint8_t),
      sizeof(uint_t)
    );
  }
}

void entitiesDispose() {
  shaderBufferDispose(&ENTITIES_BUFFER);
}