/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "map.h"
#include "util/memory.h"

shaderbuffer_t MAP_BUFFER;
mapdata_t MAP_DATA;

void mapInit() {
  memset(&MAP_DATA, 0, sizeof(mapdata_t));
  shaderBufferInit(&MAP_BUFFER, sizeof(mapdata_t));
}

void mapUpdate() {

  // Copy map size.
  memoryCopyRangeSafe(
    &MAP_DATA.mapSize,
    &OVERWORLD.mapWidth,
    &OVERWORLD.mapHeight + sizeof(uint8_t),
    sizeof(uint_t)
  );

  // Copy tile ids.
  memoryCopyRangeSafe(
    &MAP_DATA.tileIds,
    &OVERWORLD.tileIds[0],
    &OVERWORLD.tileIds[OVERWORLD.mapWidth * OVERWORLD.mapHeight],
    OVERWORLD_TILE_COUNT_MAX * sizeof(tileid_t)
  );
  
  shaderBufferBind(&MAP_BUFFER);
  shaderBufferSetData(&MAP_BUFFER, &MAP_DATA);
}

void mapDispose() {
  shaderBufferDispose(&MAP_BUFFER);
}