// Copyright (c) 2025 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "../../../../dusk/overworld/overworlddefs.h"
#include "../fragments/packed.glsl"
#include "../fragments/quad.glsl"

#define MAP_PACKED_SIZE 4

struct TileData {
  uint nothing;
  uvec3 _padding0;
};

layout(std140) uniform b_Map {
  uint mapSize;
  uint mapTileIds[OVERWORLD_TILE_COUNT_MAX / MAP_PACKED_SIZE];
  TileData mapTileData[OVERWORLD_TILE_COUNT_MAX];
  uvec3 _padding0;
};

vec2 mapTileGetSize() {
  return vec2(float(OVERWORLD_TILE_WIDTH), float(OVERWORLD_TILE_HEIGHT));
}

vec2 mapGetVertice(uint instanceIndex, uint indiceIndex) {
  vec2 quad = quadGetVertice(indiceIndex);

  uint mapWidth = packedGetU8(0u, mapSize);
  uint mapHeight = packedGetU8(1u, mapSize);

  quad += vec2(
    float(instanceIndex % mapWidth),
    float(instanceIndex / mapWidth)
  );

  quad *= mapTileGetSize();

  return quad;
}